#!/etc/opt/openit/perl

# Shuts down the currently running LSC process (supposedly hanging and running at 100% CPU usage)
# Applies the following updates to licensestatusconverter.conf:
## In licensestatusconverter.mapping, add daemon-rename and available-license-mapping
## In licensestatusconverter.output, add daemon-filtering after type
## In licensestatusconverter, add input-dir before output

my $lsc_config = "/var/opt/openit/etc/licensestatusconverter.conf";
my @lines = read_file( $lsc_config );

my @daemon_rename = (
	"\t\tdaemon-rename()=\"\"\n",
	"\t\t{\n",
	"\t\t\tfilter(String)=\"Mapping\"{}\n",
	"\t\t\tmap-file(FileName)=\"\${OpeniT.directories.conf}/daemon-rename.map\"{}\n",
	"\t\t\tmatch-string(String)=\"feature\\\\daemon\\\\type\\\\product\"{}\n",
	"\t\t\ttarget(String)=\"daemon\"{}\n",
	"\t\t\tactive(Bool)=\"false\"{}\n",
	"\t\t}\n"
	);

my @avail_license_mapping = (
	"\t\tavailable-license-mapping()=\"\"\n",
	"\t\t{\n",
	"\t\t\tfilter(String)=\"Mapping\"{}\n",
	"\t\t\tmap-file(FileName)=\"\${OpeniT.directories.conf}/available-licenses.map\"{}\n",
	"\t\t\tmatch-string(String)=\"feature\\\\daemon\\\\type\\\\product\"{}\n",
	"\t\t\ttarget(String)=\"licenses\"{}\n",
	"\t\t\tactive(Bool)=\"true\"{}\n",
	"\t\t}\n"
	);

my @daemon_filtering = (
	"\t\tdaemon-filtering()=\"\"\n",
	"\t\t{\n",
	"\t\t\tactive(Bool)=\"true\"{}\n",
	"\t\t\tmap-file(FileName)=\"\${OpeniT.directories.conf}/daemon_filter\"{}\n",
	"\t\t}\n"
	);

my $ind = 0;


# Kill existing LSC process (prevent hanging instance from blocking fix)
system( "killall -9 licensestatusconverter" );

$ind = locate_line( $ind , "licensestatusconverter" , \@lines );
$ind = locate_line( $ind , "mapping" , \@lines );
$ind = locate_line( $ind , "product-rename" , \@lines );


# Insert daemon-rename and available-license-mapping objects under mapping
if( grep( /available-license-mapping/ , @lines ) ) {
	print( "-- NOTICE: available-license-mapping configuration object already present. Skipping insertion.\n" );
}
else {
	splice @lines, $ind, 0, @avail_license_mapping;
}

if( grep( /daemon-rename/ , @lines ) ) {
	print( "-- NOTICE: daemon-rename configuration object already present. Skipping insertion.\n" );
}
else {
	splice @lines, $ind, 0, @daemon_rename;
}


# Insert input-dir object before output
if( grep( /input-dir/ , @lines ) ) {
	print( "-- NOTICE: input-dir configuration object already present. Skipping insertion.\n" );
}
else {
	$ind = locate_line( $ind , "output" , \@lines );
	$ind--;
	
	@lines[$ind] =~ s/\}\n/\}\n\tinput-dir(DirName)="\$\{OpeniT\.directories\.data\}\/license_status"\{\}\n/g;
}

# Insert daemon-filtering object under output
$ind = locate_line( $ind , "type" , \@lines );
if( grep( /daemon-filtering/ , @lines ) ) {
	print( "-- NOTICE: daemon-filtering configuration object already present. Skipping insertion.\n" );
}
else {
	splice @lines, $ind, 0, @daemon_filtering;
}


# Backup licensestatusconverter.conf and save updates
save_file( $lsc_config , 2173 , @lines );
print( "-- INFO: Done updating licensestatusconverter.conf\n" );
exit( 0 );


# Subroutines -----------

sub get_filters_to_update {
    my ( $filter_dir, $include, $exclude ) = @_;

	opendir( DIR,$filter_dir );
	my @tmpfilters = readdir( DIR );
	closedir( DIR );
	
	@tmpfilters = grep { /\.conf$/ } @tmpfilters;
	
	@tmpfilters = grep { /$include/ } @tmpfilters;
	@tmpfilters = grep { !/old/ } @tmpfilters;
	
    if( defined( $exclude ) && $exclude ne "" ) {
		@tmpfilters = grep { !/$exclude/ } @tmpfilters;
    }
	
	my @filters;

	foreach my $file ( @tmpfilters ) {
        push @filters, "$filter_dir$file";
    }
    
	return @filters;
}

sub read_file {
	my ( $cfg ) = @_;
	open( IN, "$cfg" ) || die "-- ERROR: Unable to read cfg '$cfg'!\n";
	my @lines = <IN>;
	close( IN );
	return @lines;
}

sub locate_line {
	# Unix version -- For carets and colons, use the . character to represent them
	my ( $start, $find, $lines_ref ) = (@_);
	my ( $index ) = grep { @$lines_ref[$_] =~ /$find/ } $start ..@$lines_ref;
	return $index;
}

sub save_file {
	my ( $cfg, $patch, @lines ) = @_;
	my $old = "$cfg.pre_$patch";
	my $new = "$cfg.new";
	
	open( OUT, ">$new" ) || die "-- ERROR: Can't write new cfg '$new'!\n";
	print OUT @lines;
	close( OUT );
	
	if( ! rename( $cfg, $old ) ) {
		die "-- ERROR: Unable to rename previous config '$cfg' to '$old'!\n";
	}
	
	if( ! rename( $new, $cfg ) ) {
		die "-- ERROR: Unable to rename new config to '$cfg'!\n";
	}
}
